# Vore RPG System

## Summary

This is the reference implementation for a form of text-adventure/turn-based RPG hybrid themed around the vore fetish.

In it, there are essentially four gameplay modes:

1. Adventure: The player may move between rooms (each of which has a textual description) and interact with objects or people in the current room, or use an item from their inventory on one of them.
2. Novel: Text gameplay where the player can only interact using choice menus or advancing the text, describing interactions exchanged between characters.
3. Confrontation: Even turn-based battle mode where two characters exchange actions and words with the intention of devouring each other (or simply avoiding being devoured).
4. Stomach: Uneven turn-based battle mode where two characters exchange actions and words, with the pred intending to digest the prey and prevent the prey's escape while the prey tries to resist being digested and escape.

## Objectives

This system is designed with the following opinionated principles in mind:

1. Interactive vore is a lot more fun than static vore.
2. Vore games where the gameplay isn't about engaging in or avoiding vore, but rather just a means to get between static vore scenes, are leaving a lot of opportunity on the table.
3. Reasonably well-balanced systems simplify the design of stats and allow for two-player interactions, rather than simply human vs. computer.
4. Long, slow digestion and an aftermath scene can be quite fun to indulge in, but they should not be required.
5. Making small random alterations to text can help keep the same text game a little bit fresher over multiple plays.
6. Frequent and/or automatic save games are preferred over simply undoing the effects of any vore scene after it happens.
7. Prey gameplay does well with the sensation of risk and choices where all options are unpleasant, as well as being thoroughly defeated, to the point where there is no longer anything the player can do.
8. Giving in or intentionally yielding to an opponent is usually not as much fun as losing to them after the player doing their best, but there are exceptions to this rule.
9. Pred gameplay does well with the sensation of complete domination, of taking an opponent who was a strong and independent person - maybe even a threat to the player - and breaking them down utterly. 
10. Vore is best with the intimacy of one-on-one, but turn-based RPG systems are often built around the concept of having a larger party of characters. Changes need to be made to address this.
11. Having a transcript to review the best bits of a game session is always nice.
12. Writing a vore game should preferably not require a proprietary editor to even view it, and should preferably be playable on a simple browser. 
13. This engine would be useful for more than just one game.

## Attributes

### Confidence (and Max Confidence)
A character's current mental fortitude, ranging from 0 up to (MaxConfidence * Health / MaxHealth). This means that Health and Confidence are linked, in that the maximum percentage of MaxConfidence that Confidence can be is the same as the percentage of MaxHealth that Health is.

Confidence regenerates slowly on each turn during battle. The lower it is, the slower it recovers. Characters with 0 Confidence do not recover Confidence automatically.

A character with 0 Confidence has accepted their place in the food chain, and it's not anywhere near the top. A character has a 0% chance of being swallowed (or forced to release their prey) by a finisher-type action until low Confidence, and it has a 100% chance of success at 0 Confidence.

After battle, a character who fled the battle retains their current confidence. A character who digested an opponent or caused their opponent to flee recovers all of their Confidence after the battle ends.

### Health (and Max Health)
A character's current physical vitality, ranging from -MaxHealth up to MaxHealth.

Health never recovers except at specific rest points, or when special items (and occasionally actions) are used. The prey's Health decreases gradually over time when in a stomach scene.

Other than passive digestion damage and actions by the pred during a stomach scene, most actions do not target Health. Taking damage from violent actions outside of a stomach scene has an increasing chance (starting at 50%, with a 100% chance at 0 or fewer Health) to cause a character to fall unconscious and be devoured, waking up in a stomach scene with 0 Energy.

A character at 0 Health or less is done for, they just don't know it yet. The effect of their own actions is dramatically reduced, they can no longer recover Health above 0, and they can no longer win or flee the battle, only wait for it to be over. (This waiting process can be skipped if desired.)

A character at -MaxHealth has been fully digested, and the battle finally ends.

### Energy
A character's current motivation or drive, ranging from 0 up to Stamina.

Energy recovers quickly on each turn of a battle, and is fully recovered after battle regardless of the outcome. The rate of Energy recovery in battle is proportional to the current Stamina and the number of actions used on the previous turn.

Actions have a cost in Energy, and without sufficient Energy, they can't be used.

### Stamina (and Max Stamina)
A character's current physical exhaustion level, ranging from 0 up to MaxStamina.

Stamina never recovers except at specific rest points, or when special items (and occasionally actions) are used.

Actions typically do not damage or cost Stamina; instead, Stamina is used in order to use multiple Actions in a single turn.

### Arousal (and Willpower)
A character's sense of emotional or sexual arousal, ranging from 0% to 300% + Willpower.

Arousal starts at 0% at the beginning of each battle. The higher it gets, the more power each action the character takes has, but there's also an increasing chance (starting at 100% with normal Willpower, and at the highest possible level but not a sure thing at 300%) of their action for a turn being an orgasm or other overflow of excitement, which immediately reduces their Energy to 0.

Arousal decreases slowly over each turn in a battle where it hasn't been raised within the last turn for each battle participant.

### Pressure (and Stomach Capacity)
A character's stomach pressure, ranging from 0% to 300% + StomachCapacity.

Pressure starts at 0% during a stomach scene. The higher it gets, the more damage acid does each turn, but there's also an increasing chance (starting at 100% with normal Stomach Capacity, and at the highest possible level but not a sure thing at 300%) of the pred missing their turn with a burp.

Pressure decreases slowly over each turn in a battle where it hasn't been raised within the last turn for each battle participant.

### Charisma
A character's social strength and skill with words. The character's strength with non-violent attacks.

### Patience
A character's social resilience and ability to brush social things off. The character's defense against non-violent attacks.

### Power
A character's physical strength and skill with their body or weapons. The character's strength with violent attacks.

### Toughness
A character's physical resilience and ability to deflect or reduce physical damage. The character's defense against violent attacks.